//==============================================================================
// File: 		  wnd.h
//
// Description:  this header is my interpretation of the internal WND 
//               structure of the Win32 user subsystem, at least on Win2k
//
// Revisions: 	  05/27/2000 - created
//               06/02/2000 - found out the meaning of a couple other fields 
//
//==============================================================================
// Tomas Restrepo
// Send comments to: tomasr@mvps.org
//==============================================================================

#ifndef _WND_H_
#define _WND_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

//
// internal window class representations
//
typedef struct CLASSINFO
{
    DWORD       unk1;
    ATOM        atomWindowType;
    WORD        unk2;
    DWORD       unk3;
    DWORD       unk4;
    DWORD       unk5;
    DWORD       unk6;
    DWORD       unk7;
    DWORD       unk8;
    CLASSINFO*  pSelf;
    DWORD       unk10;
    DWORD       unk11;
    DWORD       dwStyle;
    DWORD       unk13;
    DWORD       unk14;
    DWORD       dwClassBytes;
    HINSTANCE   hInstance;
} CLASSINFO, *PCLASSINFO;
struct WNDMENU;

// 
// possible flag values for dwFlags
//
#define WF_ACTIVECAPTION  (0x00000040)
#define WF_ANSI           (0x00080000)


// 
// All pointers that appear in this struct point to memory
// above the 2GB limit, deep into System Address Space.
// However, like the WND structures themselves, all of the
// data pointed to by them can be found in the Process
// address space by substracting from them the same amount
// we use to find this WND*
//
typedef struct WND {
      HWND        hWnd;
      ULONG       unk1;
      ULONG       unk2;
      ULONG       unk3;
      WND*        pSelf;         // pointer to self 
      DWORD       dwFlags;
      ULONG       unk6;
      DWORD       dwStyleEx;
      DWORD       dwStyle;
      HINSTANCE   hInstance;
      ULONG       unk10;
      WND*        pNextWnd;      // pointer to "next" window 
      WND*        pParentWnd;    // pointer to parent wnd. For top-level, this points to the desktop
      WND*        pFirstChild;   // pointer to first child window 
      WND*        pOwnerWnd;     // pointer to owner window 
      RECT        rcWnd;         // in screen coordinates
      RECT        rcClient;      // in screen coordinates
      WNDPROC*    pWndProc;      // for system classes, this can be > 2GB
      CLASSINFO*  pWndClass;     // pointer to internal window class ???
      ULONG       unk25;
      ULONG       unk26;
      ULONG       unk27;
      ULONG       unk28;
      union {
         DWORD       dwWndID;    // control ID (lower 16-bits)
         WNDMENU*    pMenu;      // pointer to internal menu representation
      } id;
      ULONG       unk30;
      ULONG       unk31;
      ULONG       unk32;
      WCHAR*      pText;         // pointer to window caption
      DWORD       dwWndBytes;    // number of window bytes reserved in wlWndExtra
      ULONG       unk35;         // for top level windows, this is pSelf again....???
      ULONG       unk36;
      ULONG       wlUserData;
      ULONG       wlWndExtra[1];
} WND, *PWND;

#ifdef __cplusplus
} 
#endif // __cplusplus

#endif // _WND_H_
